// Copyright (C) INRIA
// Copyright (C) 2010 - DIGITEO - Allan CORNET
// Copyright (C) 2010 - DIGITEO - Michael Baudin
// Copyright (C) 2011 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// Headers from Scilab:
#include "sciprint.h"
#include "Scierror.h"
#include "machine.h"
#include "core_math.h"
#include "api_scilab.h"
#include "stack-c.h"

// Headers from the gateway:
#include "gw_linalg.h"
#include "gw_linalg_support.h"


/* -------------------------------------------------------------------------- */
/* SUBROUTINE DSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO ) */
/* [R,d] = dsyev(A) */
/* d = dsyev(A) */
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
int sci_linalg_dsyev(char* fname)
{
	int M = 0, N = 0;
	int NLHS = 0;
	int un = 1;
	int WORK = 0, LWORKMIN = 0, LWORK = 0, INFO = 0;
	int LDA = 0;

	int *piAddr = NULL;
	SciErr sciErr;
	int iRet = 0;
	double * lrA;
	double * lrd;
	double * lrWORK;

	static int minlhs=1, minrhs=1, maxlhs=2, maxrhs=1;
	CheckRhs(minrhs,maxrhs) ;  
	CheckLhs(minlhs,maxlhs) ;

	// Get A
	sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 1;
	}
	iRet = getRealMatrixOfDoubles( fname, 1 , piAddr , &lrA, &M , &N );
	if(iRet)
	{
		return 1;
	}
	CheckSquare(1,M,N);
	//
	// Create d
	sciErr = allocMatrixOfDouble(pvApiCtx, 2, N, 1, &lrd);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
	//
	// Create a workspace
	WORK=3;
	LWORKMIN = Max(1, 3*N-1);
	LWORK=C2F(maxvol)(&WORK, "d", 1L);  /* max memory currently available */
	if (LWORK < LWORKMIN) {
		Scierror(9999,"%s: Not enough memory (use stacksize)",fname);
		return 0;
	};
	sciErr = allocMatrixOfDouble(pvApiCtx, 3, LWORK, 1, &lrWORK);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
	LDA = Max(1,N);

	if ( Lhs == 2)
	{
		/*  [R,d]=dsyev(A)     */

		C2F(dsyev)("V", "L", &N, lrA, &LDA, lrd, lrWORK, &LWORK, &INFO);
		if (INFO != 0) {
			C2F(errorinfo)("dsyev ", &INFO, 5L);
			return 0;
		}
		LhsVar(1)=1; 
		LhsVar(2)=2;

	}
	else
	{
		/*   d=dsyev(A)    */

		C2F(dsyev)("N", "L", &N, lrA, &LDA, lrd, lrWORK, &LWORK, &INFO);
		if (INFO != 0) {
			C2F(errorinfo)("dsyev ", &INFO, 5L);
			return 0;
		}
		LhsVar(1)=2;

	}
	return 0;
}
/* -------------------------------------------------------------------------- */
